@@ -11,19 +11,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import datetime
 import json
-import ssl
-import sys
 import unittest
 
+import fixtures
 import mock
+import oslo_messaging as messaging
 from oslo_utils import uuidutils
 
 from ironic_inspector.common import ironic as ir_utils
-from ironic_inspector import conf
-from ironic_inspector import db
-from ironic_inspector import firewall
-from ironic_inspector import introspect
+from ironic_inspector.common import rpc
+import ironic_inspector.conf
+from ironic_inspector.conf import opts as conf_opts
+from ironic_inspector import introspection_state as istate
 from ironic_inspector import main
 from ironic_inspector import node_cache
 from ironic_inspector.plugins import base as plugins_base
@@ -32,9 +33,8 @@ from ironic_inspector import process
 from ironic_inspector import rules
 from ironic_inspector.test import base as test_base
 from ironic_inspector import utils
-from oslo_config import cfg
 
-CONF = cfg.CONF
+CONF = ironic_inspector.conf.CONF
 
 
 def _get_error(res):
@@ -51,58 +51,76 @@ class BaseAPITest(test_base.BaseTest):
 
 
 class TestApiIntrospect(BaseAPITest):
-    @mock.patch.object(introspect, 'introspect', autospec=True)
-    def test_introspect_no_authentication(self, introspect_mock):
+    def setUp(self):
+        super(TestApiIntrospect, self).setUp()
+        self.rpc_get_client_mock = self.useFixture(
+            fixtures.MockPatchObject(rpc, 'get_client', autospec=True)).mock
+        self.client_mock = mock.MagicMock(spec=messaging.RPCClient)
+        self.rpc_get_client_mock.return_value = self.client_mock
+
+    def test_introspect_no_authentication(self):
         CONF.set_override('auth_strategy', 'noauth')
+
         res = self.app.post('/v1/introspection/%s' % self.uuid)
-        self.assertEqual(202, res.status_code)
-        introspect_mock.assert_called_once_with(self.uuid,
-                                                new_ipmi_credentials=None,
-                                                token=None)
 
-    @mock.patch.object(introspect, 'introspect', autospec=True)
-    def test_introspect_set_ipmi_credentials(self, introspect_mock):
-        res = self.app.post('/v1/introspection/%s?new_ipmi_username=user&'
-                            'new_ipmi_password=password' % self.uuid)
         self.assertEqual(202, res.status_code)
-        introspect_mock.assert_called_once_with(
-            self.uuid,
-            new_ipmi_credentials=('user', 'password'),
-            token=None)
+        self.client_mock.call.assert_called_once_with({}, 'do_introspection',
+                                                      node_id=self.uuid,
+                                                      manage_boot=True,
+                                                      token=None)
 
-    @mock.patch.object(introspect, 'introspect', autospec=True)
-    def test_introspect_set_ipmi_credentials_no_user(self, introspect_mock):
-        res = self.app.post('/v1/introspection/%s?'
-                            'new_ipmi_password=password' % self.uuid)
-        self.assertEqual(202, res.status_code)
-        introspect_mock.assert_called_once_with(
-            self.uuid,
-            new_ipmi_credentials=(None, 'password'),
-            token=None)
-
-    @mock.patch.object(introspect, 'introspect', autospec=True)
-    def test_intospect_failed(self, introspect_mock):
-        introspect_mock.side_effect = utils.Error("boom")
+    def test_intospect_failed(self):
+        self.client_mock.call.side_effect = utils.Error("boom")
         res = self.app.post('/v1/introspection/%s' % self.uuid)
+
         self.assertEqual(400, res.status_code)
         self.assertEqual(
             'boom',
             json.loads(res.data.decode('utf-8'))['error']['message'])
-        introspect_mock.assert_called_once_with(
-            self.uuid,
-            new_ipmi_credentials=None,
-            token=None)
+        self.client_mock.call.assert_called_once_with({}, 'do_introspection',
+                                                      node_id=self.uuid,
+                                                      manage_boot=True,
+                                                      token=None)
+
+    def test_introspect_no_manage_boot(self):
+        res = self.app.post('/v1/introspection/%s?manage_boot=0' % self.uuid)
+        self.assertEqual(202, res.status_code)
+        self.client_mock.call.assert_called_once_with({}, 'do_introspection',
+                                                      node_id=self.uuid,
+                                                      manage_boot=False,
+                                                      token=None)
+
+    def test_introspect_can_manage_boot_false(self):
+        CONF.set_override('can_manage_boot', False)
+        res = self.app.post('/v1/introspection/%s?manage_boot=0' % self.uuid)
+        self.assertEqual(202, res.status_code)
+        self.client_mock.call.assert_called_once_with({}, 'do_introspection',
+                                                      node_id=self.uuid,
+                                                      manage_boot=False,
+                                                      token=None)
+
+    def test_introspect_can_manage_boot_false_failed(self):
+        CONF.set_override('can_manage_boot', False)
+        res = self.app.post('/v1/introspection/%s' % self.uuid)
+        self.assertEqual(400, res.status_code)
+        self.assertFalse(self.client_mock.call.called)
+
+    def test_introspect_wrong_manage_boot(self):
+        res = self.app.post('/v1/introspection/%s?manage_boot=foo' % self.uuid)
+        self.assertEqual(400, res.status_code)
+        self.assertFalse(self.client_mock.call.called)
+        self.assertEqual(
+            'Invalid boolean value for manage_boot: foo',
+            json.loads(res.data.decode('utf-8'))['error']['message'])
 
     @mock.patch.object(utils, 'check_auth', autospec=True)
-    @mock.patch.object(introspect, 'introspect', autospec=True)
-    def test_introspect_failed_authentication(self, introspect_mock,
-                                              auth_mock):
+    def test_introspect_failed_authentication(self, auth_mock):
         CONF.set_override('auth_strategy', 'keystone')
         auth_mock.side_effect = utils.Error('Boom', code=403)
         res = self.app.post('/v1/introspection/%s' % self.uuid,
                             headers={'X-Auth-Token': 'token'})
         self.assertEqual(403, res.status_code)
-        self.assertFalse(introspect_mock.called)
+        self.assertFalse(self.client_mock.call.called)
 
 
 @mock.patch.object(process, 'process', autospec=True)
@@ -131,72 +149,153 @@ class TestApiContinue(BaseAPITest):
         self.assertFalse(process_mock.called)
 
 
-@mock.patch.object(introspect, 'abort', autospec=True)
 class TestApiAbort(BaseAPITest):
-    def test_ok(self, abort_mock):
-        abort_mock.return_value = '', 202
+    def setUp(self):
+        super(TestApiAbort, self).setUp()
+        self.rpc_get_client_mock = self.useFixture(
+            fixtures.MockPatchObject(rpc, 'get_client', autospec=True)).mock
+        self.client_mock = mock.MagicMock(spec=messaging.RPCClient)
+        self.rpc_get_client_mock.return_value = self.client_mock
+
+    def test_ok(self):
 
         res = self.app.post('/v1/introspection/%s/abort' % self.uuid,
                             headers={'X-Auth-Token': 'token'})
 
-        abort_mock.assert_called_once_with(self.uuid, token='token')
+        self.client_mock.call.assert_called_once_with({}, 'do_abort',
+                                                      node_id=self.uuid,
+                                                      token='token')
         self.assertEqual(202, res.status_code)
         self.assertEqual(b'', res.data)
 
-    def test_no_authentication(self, abort_mock):
-        abort_mock.return_value = b'', 202
+    def test_no_authentication(self):
 
         res = self.app.post('/v1/introspection/%s/abort' % self.uuid)
 
-        abort_mock.assert_called_once_with(self.uuid, token=None)
+        self.client_mock.call.assert_called_once_with({}, 'do_abort',
+                                                      node_id=self.uuid,
+                                                      token=None)
         self.assertEqual(202, res.status_code)
         self.assertEqual(b'', res.data)
 
-    def test_node_not_found(self, abort_mock):
+    def test_node_not_found(self):
         exc = utils.Error("Not Found.", code=404)
-        abort_mock.side_effect = exc
+        self.client_mock.call.side_effect = exc
 
         res = self.app.post('/v1/introspection/%s/abort' % self.uuid)
 
-        abort_mock.assert_called_once_with(self.uuid, token=None)
+        self.client_mock.call.assert_called_once_with({}, 'do_abort',
+                                                      node_id=self.uuid,
+                                                      token=None)
         self.assertEqual(404, res.status_code)
         data = json.loads(str(res.data.decode()))
         self.assertEqual(str(exc), data['error']['message'])
 
-    def test_abort_failed(self, abort_mock):
+    def test_abort_failed(self):
         exc = utils.Error("Locked.", code=409)
-        abort_mock.side_effect = exc
+        self.client_mock.call.side_effect = exc
 
         res = self.app.post('/v1/introspection/%s/abort' % self.uuid)
 
-        abort_mock.assert_called_once_with(self.uuid, token=None)
+        self.client_mock.call.assert_called_once_with({}, 'do_abort',
+                                                      node_id=self.uuid,
+                                                      token=None)
         self.assertEqual(409, res.status_code)
         data = json.loads(res.data.decode())
         self.assertEqual(str(exc), data['error']['message'])
 
 
-class TestApiGetStatus(BaseAPITest):
-    @mock.patch.object(node_cache, 'get_node', autospec=True)
+class GetStatusAPIBaseTest(BaseAPITest):
+    def setUp(self):
+        super(GetStatusAPIBaseTest, self).setUp()
+        self.uuid2 = uuidutils.generate_uuid()
+        self.finished_node = node_cache.NodeInfo(
+            uuid=self.uuid,
+            started_at=datetime.datetime(1, 1, 1),
+            finished_at=datetime.datetime(1, 1, 2),
+            error='boom',
+            state=istate.States.error)
+        self.finished_node.links = [
+            {u'href': u'http://localhost/v1/introspection/%s' %
+             self.finished_node.uuid,
+             u'rel': u'self'},
+        ]
+        self.finished_node.status = {
+            'finished': True,
+            'state': self.finished_node._state,
+            'started_at': self.finished_node.started_at.isoformat(),
+            'finished_at': self.finished_node.finished_at.isoformat(),
+            'error': self.finished_node.error,
+            'uuid': self.finished_node.uuid,
+            'links': self.finished_node.links
+        }
+
+        self.unfinished_node = node_cache.NodeInfo(
+            uuid=self.uuid2,
+            started_at=datetime.datetime(1, 1, 1),
+            state=istate.States.processing)
+        self.unfinished_node.links = [
+            {u'href': u'http://localhost/v1/introspection/%s' %
+             self.unfinished_node.uuid,
+             u'rel': u'self'}
+        ]
+        finished_at = (self.unfinished_node.finished_at.isoformat()
+                       if self.unfinished_node.finished_at else None)
+        self.unfinished_node.status = {
+            'finished': False,
+            'state': self.unfinished_node._state,
+            'started_at': self.unfinished_node.started_at.isoformat(),
+            'finished_at': finished_at,
+            'error': None,
+            'uuid': self.unfinished_node.uuid,
+            'links': self.unfinished_node.links
+        }
+
+
+@mock.patch.object(node_cache, 'get_node', autospec=True)
+class TestApiGetStatus(GetStatusAPIBaseTest):
     def test_get_introspection_in_progress(self, get_mock):
-        get_mock.return_value = node_cache.NodeInfo(uuid=self.uuid,
-                                                    started_at=42.0)
+        get_mock.return_value = self.unfinished_node
         res = self.app.get('/v1/introspection/%s' % self.uuid)
         self.assertEqual(200, res.status_code)
-        self.assertEqual({'finished': False, 'error': None},
+        self.assertEqual(self.unfinished_node.status,
                          json.loads(res.data.decode('utf-8')))
 
-    @mock.patch.object(node_cache, 'get_node', autospec=True)
     def test_get_introspection_finished(self, get_mock):
-        get_mock.return_value = node_cache.NodeInfo(uuid=self.uuid,
-                                                    started_at=42.0,
-                                                    finished_at=100.1,
-                                                    error='boom')
+        get_mock.return_value = self.finished_node
         res = self.app.get('/v1/introspection/%s' % self.uuid)
         self.assertEqual(200, res.status_code)
-        self.assertEqual({'finished': True, 'error': 'boom'},
+        self.assertEqual(self.finished_node.status,
                          json.loads(res.data.decode('utf-8')))
 
 
+@mock.patch.object(node_cache, 'get_node_list', autospec=True)
+class TestApiListStatus(GetStatusAPIBaseTest):
+
+    def test_list_introspection(self, list_mock):
+        list_mock.return_value = [self.finished_node, self.unfinished_node]
+        res = self.app.get('/v1/introspection')
+        self.assertEqual(200, res.status_code)
+        statuses = json.loads(res.data.decode('utf-8')).get('introspection')
+
+        self.assertEqual([self.finished_node.status,
+                          self.unfinished_node.status], statuses)
+        list_mock.assert_called_once_with(marker=None,
+                                          limit=CONF.api_max_limit)
+
+    def test_list_introspection_limit(self, list_mock):
+        res = self.app.get('/v1/introspection?limit=1000')
+        self.assertEqual(200, res.status_code)
+        list_mock.assert_called_once_with(marker=None, limit=1000)
+
+    def test_list_introspection_makrer(self, list_mock):
+        res = self.app.get('/v1/introspection?marker=%s' %
+                           self.finished_node.uuid)
+        self.assertEqual(200, res.status_code)
+        list_mock.assert_called_once_with(marker=self.finished_node.uuid,
+                                          limit=CONF.api_max_limit)
+
+
 class TestApiGetData(BaseAPITest):
     @mock.patch.object(main.swift, 'SwiftAPI', autospec=True)
     def test_get_introspection_data(self, swift_mock):
@@ -252,29 +351,33 @@ class TestApiGetData(BaseAPITest):
         get_mock.assert_called_once_with('name1', fields=['uuid'])
 
 
-@mock.patch.object(process, 'reapply', autospec=True)
 class TestApiReapply(BaseAPITest):
 
     def setUp(self):
         super(TestApiReapply, self).setUp()
+        self.rpc_get_client_mock = self.useFixture(
+            fixtures.MockPatchObject(rpc, 'get_client', autospec=True)).mock
+        self.client_mock = mock.MagicMock(spec=messaging.RPCClient)
+        self.rpc_get_client_mock.return_value = self.client_mock
         CONF.set_override('store_data', 'swift', 'processing')
 
-    def test_ok(self, reapply_mock):
+    def test_ok(self):
 
         self.app.post('/v1/introspection/%s/data/unprocessed' %
                       self.uuid)
-        reapply_mock.assert_called_once_with(self.uuid)
+        self.client_mock.call.assert_called_once_with({}, 'do_reapply',
+                                                      node_id=self.uuid)
 
-    def test_user_data(self, reapply_mock):
+    def test_user_data(self):
         res = self.app.post('/v1/introspection/%s/data/unprocessed' %
                             self.uuid, data='some data')
         self.assertEqual(400, res.status_code)
         message = json.loads(res.data.decode())['error']['message']
         self.assertEqual('User data processing is not supported yet',
                          message)
-        self.assertFalse(reapply_mock.called)
+        self.assertFalse(self.client_mock.call.called)
 
-    def test_swift_disabled(self, reapply_mock):
+    def test_swift_disabled(self):
         CONF.set_override('store_data', 'none', 'processing')
 
         res = self.app.post('/v1/introspection/%s/data/unprocessed' %
@@ -285,35 +388,11 @@ class TestApiReapply(BaseAPITest):
                          'data. Set the [processing] store_data '
                          'configuration option to change this.',
                          message)
-        self.assertFalse(reapply_mock.called)
+        self.assertFalse(self.client_mock.call.called)
 
-    def test_node_locked(self, reapply_mock):
-        exc = utils.Error('Locked.', code=409)
-        reapply_mock.side_effect = exc
-
-        res = self.app.post('/v1/introspection/%s/data/unprocessed' %
-                            self.uuid)
-
-        self.assertEqual(409, res.status_code)
-        message = json.loads(res.data.decode())['error']['message']
-        self.assertEqual(str(exc), message)
-        reapply_mock.assert_called_once_with(self.uuid)
-
-    def test_node_not_found(self, reapply_mock):
-        exc = utils.Error('Not found.', code=404)
-        reapply_mock.side_effect = exc
-
-        res = self.app.post('/v1/introspection/%s/data/unprocessed' %
-                            self.uuid)
-
-        self.assertEqual(404, res.status_code)
-        message = json.loads(res.data.decode())['error']['message']
-        self.assertEqual(str(exc), message)
-        reapply_mock.assert_called_once_with(self.uuid)
-
-    def test_generic_error(self, reapply_mock):
+    def test_generic_error(self):
         exc = utils.Error('Oops', code=400)
-        reapply_mock.side_effect = exc
+        self.client_mock.call.side_effect = exc
 
         res = self.app.post('/v1/introspection/%s/data/unprocessed' %
                             self.uuid)
@@ -321,7 +400,8 @@ class TestApiReapply(BaseAPITest):
         self.assertEqual(400, res.status_code)
         message = json.loads(res.data.decode())['error']['message']
         self.assertEqual(str(exc), message)
-        reapply_mock.assert_called_once_with(self.uuid)
+        self.client_mock.call.assert_called_once_with({}, 'do_reapply',
+                                                      node_id=self.uuid)
 
 
 class TestApiRules(BaseAPITest):
@@ -386,7 +466,7 @@ class TestApiRules(BaseAPITest):
         create_mock.return_value = mock.Mock(spec=rules.IntrospectionRule,
                                              **{'as_dict.return_value': exp})
 
-        headers = {conf.VERSION_HEADER:
+        headers = {conf_opts.VERSION_HEADER:
                    main._format_version((1, 5))}
 
         res = self.app.post('/v1/rules', data=json.dumps(data),
@@ -465,9 +545,9 @@ class TestApiMisc(BaseAPITest):
 class TestApiVersions(BaseAPITest):
     def _check_version_present(self, res):
         self.assertEqual('%d.%d' % main.MINIMUM_API_VERSION,
-                         res.headers.get(conf.MIN_VERSION_HEADER))
+                         res.headers.get(conf_opts.MIN_VERSION_HEADER))
         self.assertEqual('%d.%d' % main.CURRENT_API_VERSION,
-                         res.headers.get(conf.MAX_VERSION_HEADER))
+                         res.headers.get(conf_opts.MAX_VERSION_HEADER))
 
     def test_root_endpoint(self):
         res = self.app.get("/")
@@ -510,7 +590,7 @@ class TestApiVersions(BaseAPITest):
                     u'href': u'http://localhost/v1/endpoint3'}]
             },
         ]}
-        self.assertDictEqual(expected, json_data)
+        self.assertEqual(expected, json_data)
 
     def test_version_endpoint_invalid(self):
         endpoint = "/v-1"
@@ -521,11 +601,14 @@ class TestApiVersions(BaseAPITest):
         # API version on unknown pages
         self._check_version_present(self.app.get('/v1/foobar'))
 
+    @mock.patch.object(rpc, 'get_client', autospec=True)
     @mock.patch.object(node_cache, 'get_node', autospec=True)
-    def test_usual_requests(self, get_mock):
+    def test_usual_requests(self, get_mock, rpc_mock):
+        client_mock = mock.MagicMock(spec=messaging.RPCClient)
+        rpc_mock.return_value = client_mock
         get_mock.return_value = node_cache.NodeInfo(uuid=self.uuid,
                                                     started_at=42.0)
-        # Successfull
+        # Successful
         self._check_version_present(
             self.app.post('/v1/introspection/%s' % self.uuid))
         # With error
@@ -533,14 +616,14 @@ class TestApiVersions(BaseAPITest):
             self.app.post('/v1/introspection/foobar'))
 
     def test_request_correct_version(self):
-        headers = {conf.VERSION_HEADER:
+        headers = {conf_opts.VERSION_HEADER:
                    main._format_version(main.CURRENT_API_VERSION)}
         self._check_version_present(self.app.get('/', headers=headers))
 
     def test_request_unsupported_version(self):
         bad_version = (main.CURRENT_API_VERSION[0],
                        main.CURRENT_API_VERSION[1] + 1)
-        headers = {conf.VERSION_HEADER:
+        headers = {conf_opts.VERSION_HEADER:
                    main._format_version(bad_version)}
         res = self.app.get('/', headers=headers)
         self._check_version_present(res)
@@ -568,154 +651,3 @@ class TestPlugins(unittest.TestCase):
     def test_manager_is_cached(self):
         self.assertIs(plugins_base.processing_hooks_manager(),
                       plugins_base.processing_hooks_manager())
-
-
-@mock.patch.object(firewall, 'init')
-@mock.patch.object(utils, 'add_auth_middleware')
-@mock.patch.object(ir_utils, 'get_client')
-@mock.patch.object(db, 'init')
-class TestInit(test_base.BaseTest):
-    def setUp(self):
-        super(TestInit, self).setUp()
-        # Tests default to a synchronous executor which can't be used here
-        utils._EXECUTOR = None
-        self.service = main.Service()
-
-    @mock.patch.object(firewall, 'clean_up', lambda: None)
-    def tearDown(self):
-        self.service.shutdown()
-        super(TestInit, self).tearDown()
-
-    def test_ok(self, mock_node_cache, mock_get_client, mock_auth,
-                mock_firewall):
-        CONF.set_override('auth_strategy', 'keystone')
-        self.service.init()
-        mock_auth.assert_called_once_with(main.app)
-        mock_node_cache.assert_called_once_with()
-        mock_firewall.assert_called_once_with()
-
-    def test_init_without_authenticate(self, mock_node_cache, mock_get_client,
-                                       mock_auth, mock_firewall):
-        CONF.set_override('auth_strategy', 'noauth')
-        self.service.init()
-        self.assertFalse(mock_auth.called)
-
-    @mock.patch.object(main.LOG, 'warning')
-    def test_init_with_no_data_storage(self, mock_log, mock_node_cache,
-                                       mock_get_client, mock_auth,
-                                       mock_firewall):
-        msg = ('Introspection data will not be stored. Change '
-               '"[processing] store_data" option if this is not the '
-               'desired behavior')
-        self.service.init()
-        mock_log.assert_called_once_with(msg)
-
-    @mock.patch.object(main.LOG, 'info')
-    def test_init_with_swift_storage(self, mock_log, mock_node_cache,
-                                     mock_get_client, mock_auth,
-                                     mock_firewall):
-        CONF.set_override('store_data', 'swift', 'processing')
-        msg = mock.call('Introspection data will be stored in Swift in the '
-                        'container %s', CONF.swift.container)
-        self.service.init()
-        self.assertIn(msg, mock_log.call_args_list)
-
-    def test_init_without_manage_firewall(self, mock_node_cache,
-                                          mock_get_client, mock_auth,
-                                          mock_firewall):
-        CONF.set_override('manage_firewall', False, 'firewall')
-        self.service.init()
-        self.assertFalse(mock_firewall.called)
-
-    @mock.patch.object(main.LOG, 'critical')
-    def test_init_failed_processing_hook(self, mock_log, mock_node_cache,
-                                         mock_get_client, mock_auth,
-                                         mock_firewall):
-        CONF.set_override('processing_hooks', 'foo!', 'processing')
-        plugins_base._HOOKS_MGR = None
-
-        self.assertRaises(SystemExit, self.service.init)
-        mock_log.assert_called_once_with(mock.ANY, "'foo!'")
-
-
-class TestCreateSSLContext(test_base.BaseTest):
-
-    def test_use_ssl_false(self):
-        CONF.set_override('use_ssl', False)
-        con = main.create_ssl_context()
-        self.assertIsNone(con)
-
-    @mock.patch.object(sys, 'version_info')
-    def test_old_python_returns_none(self, mock_version_info):
-        mock_version_info.__lt__.return_value = True
-        CONF.set_override('use_ssl', True)
-        con = main.create_ssl_context()
-        self.assertIsNone(con)
-
-    @unittest.skipIf(sys.version_info[:3] < (2, 7, 9),
-                     'This feature is unsupported in this version of python '
-                     'so the tests will be skipped')
-    @mock.patch.object(ssl, 'create_default_context', autospec=True)
-    def test_use_ssl_true(self, mock_cdc):
-        CONF.set_override('use_ssl', True)
-        m_con = mock_cdc()
-        con = main.create_ssl_context()
-        self.assertEqual(m_con, con)
-
-    @unittest.skipIf(sys.version_info[:3] < (2, 7, 9),
-                     'This feature is unsupported in this version of python '
-                     'so the tests will be skipped')
-    @mock.patch.object(ssl, 'create_default_context', autospec=True)
-    def test_only_key_path_provided(self, mock_cdc):
-        CONF.set_override('use_ssl', True)
-        CONF.set_override('ssl_key_path', '/some/fake/path')
-        mock_context = mock_cdc()
-        con = main.create_ssl_context()
-        self.assertEqual(mock_context, con)
-        self.assertFalse(mock_context.load_cert_chain.called)
-
-    @unittest.skipIf(sys.version_info[:3] < (2, 7, 9),
-                     'This feature is unsupported in this version of python '
-                     'so the tests will be skipped')
-    @mock.patch.object(ssl, 'create_default_context', autospec=True)
-    def test_only_cert_path_provided(self, mock_cdc):
-        CONF.set_override('use_ssl', True)
-        CONF.set_override('ssl_cert_path', '/some/fake/path')
-        mock_context = mock_cdc()
-        con = main.create_ssl_context()
-        self.assertEqual(mock_context, con)
-        self.assertFalse(mock_context.load_cert_chain.called)
-
-    @unittest.skipIf(sys.version_info[:3] < (2, 7, 9),
-                     'This feature is unsupported in this version of python '
-                     'so the tests will be skipped')
-    @mock.patch.object(ssl, 'create_default_context', autospec=True)
-    def test_both_paths_provided(self, mock_cdc):
-        key_path = '/some/fake/path/key'
-        cert_path = '/some/fake/path/cert'
-        CONF.set_override('use_ssl', True)
-        CONF.set_override('ssl_key_path', key_path)
-        CONF.set_override('ssl_cert_path', cert_path)
-        mock_context = mock_cdc()
-        con = main.create_ssl_context()
-        self.assertEqual(mock_context, con)
-        mock_context.load_cert_chain.assert_called_once_with(cert_path,
-                                                             key_path)
-
-    @unittest.skipIf(sys.version_info[:3] < (2, 7, 9),
-                     'This feature is unsupported in this version of python '
-                     'so the tests will be skipped')
-    @mock.patch.object(ssl, 'create_default_context', autospec=True)
-    def test_load_cert_chain_fails(self, mock_cdc):
-        CONF.set_override('use_ssl', True)
-        key_path = '/some/fake/path/key'
-        cert_path = '/some/fake/path/cert'
-        CONF.set_override('use_ssl', True)
-        CONF.set_override('ssl_key_path', key_path)
-        CONF.set_override('ssl_cert_path', cert_path)
-        mock_context = mock_cdc()
-        mock_context.load_cert_chain.side_effect = IOError('Boom!')
-        con = main.create_ssl_context()
-        self.assertEqual(mock_context, con)
-        mock_context.load_cert_chain.assert_called_once_with(cert_path,
-                                                             key_path)
