@@ -24,21 +24,24 @@ The test will then use that db and u/p combo to run the tests.
 
 
 import contextlib
+import datetime
 
 import alembic
 from alembic import script
 import mock
 from oslo_config import cfg
 from oslo_db.sqlalchemy.migration_cli import ext_alembic
+from oslo_db.sqlalchemy import orm
 from oslo_db.sqlalchemy import test_base
 from oslo_db.sqlalchemy import test_migrations
 from oslo_db.sqlalchemy import utils as db_utils
 from oslo_log import log as logging
+from oslo_utils import uuidutils
 import sqlalchemy
 
-from ironic_inspector.common.i18n import _LE
 from ironic_inspector import db
 from ironic_inspector import dbsync
+from ironic_inspector import introspection_state as istate
 from ironic_inspector.test import base
 
 CONF = cfg.CONF
@@ -80,18 +83,12 @@ def _is_backend_avail(backend, user, passwd, database):
         return True
 
 
-class FakeFacade(object):
-    def __init__(self, engine):
-        self.engine = engine
-
-    def get_engine(self):
-        return self.engine
-
-
 @contextlib.contextmanager
 def patch_with_engine(engine):
-    with mock.patch.object(db, 'create_facade_lazily') as patch_engine:
-        patch_engine.return_value = FakeFacade(engine)
+    with mock.patch.object(db, 'get_writer_session') as patch_w_sess, \
+            mock.patch.object(db, 'get_reader_session') as patch_r_sess:
+        patch_w_sess.return_value = patch_r_sess.return_value = (
+            orm.get_maker(engine)())
         yield
 
 
@@ -137,8 +134,8 @@ class WalkVersionsMixin(object):
                 if check:
                     check(engine, data)
         except Exception:
-            LOG.error(_LE("Failed to migrate to version %(version)s on engine "
-                          "%(engine)s"),
+            LOG.error("Failed to migrate to version %(version)s on engine "
+                      "%(engine)s",
                       {'version': version, 'engine': engine})
             raise
 
@@ -314,6 +311,136 @@ class MigrationCheckersMixin(object):
             rule_conditions.c.id == 2).execute().first()
         self.assertTrue(conds['invert'])
 
+    def _pre_upgrade_d2e48801c8ef(self, engine):
+        ok_node_id = uuidutils.generate_uuid()
+        err_node_id = uuidutils.generate_uuid()
+        data = [
+            {
+                'uuid': ok_node_id,
+                'error': None,
+                'finished_at': 0.0,
+                'started_at': 0.0
+            },
+            {
+                'uuid': err_node_id,
+                'error': 'Oops!',
+                'finished_at': 0.0,
+                'started_at': 0.0
+            }
+        ]
+        nodes = db_utils.get_table(engine, 'nodes')
+        for node in data:
+            nodes.insert().execute(node)
+        return {'err_node_id': err_node_id, 'ok_node_id': ok_node_id}
+
+    def _check_d2e48801c8ef(self, engine, data):
+        nodes = db_utils.get_table(engine, 'nodes')
+        col_names = [column.name for column in nodes.c]
+        self.assertIn('uuid', col_names)
+        self.assertIsInstance(nodes.c.uuid.type, sqlalchemy.types.String)
+        self.assertIn('version_id', col_names)
+        self.assertIsInstance(nodes.c.version_id.type, sqlalchemy.types.String)
+        self.assertIn('state', col_names)
+        self.assertIsInstance(nodes.c.state.type, sqlalchemy.types.String)
+        self.assertIn('started_at', col_names)
+        self.assertIsInstance(nodes.c.started_at.type, sqlalchemy.types.Float)
+        self.assertIn('finished_at', col_names)
+        self.assertIsInstance(nodes.c.started_at.type, sqlalchemy.types.Float)
+        self.assertIn('error', col_names)
+        self.assertIsInstance(nodes.c.error.type, sqlalchemy.types.Text)
+
+        ok_node_id = data['ok_node_id']
+        err_node_id = data['err_node_id']
+        # assert the ok node is in the (default) finished state
+        ok_node = nodes.select(nodes.c.uuid == ok_node_id).execute().first()
+        self.assertEqual(istate.States.finished, ok_node['state'])
+        # assert err node state is error after the migration
+        # even though the default state is finished
+        err_node = nodes.select(nodes.c.uuid == err_node_id).execute().first()
+        self.assertEqual(istate.States.error, err_node['state'])
+
+    def _pre_upgrade_d00d6e3f38c4(self, engine):
+        nodes = db_utils.get_table(engine, 'nodes')
+        data = []
+        for finished_at in (None, 1234.0):
+            node = {'uuid': uuidutils.generate_uuid(),
+                    'started_at': 1232.0,
+                    'finished_at': finished_at,
+                    'error': None}
+            nodes.insert().values(node).execute()
+            data.append(node)
+        return data
+
+    def _check_d00d6e3f38c4(self, engine, data):
+        nodes = db_utils.get_table(engine, 'nodes')
+        col_names = [column.name for column in nodes.c]
+
+        self.assertIn('started_at', col_names)
+        self.assertIn('finished_at', col_names)
+        self.assertIsInstance(nodes.c.started_at.type,
+                              sqlalchemy.types.DateTime)
+        self.assertIsInstance(nodes.c.finished_at.type,
+                              sqlalchemy.types.DateTime)
+
+        for node in data:
+            finished_at = datetime.datetime.utcfromtimestamp(
+                node['finished_at']) if node['finished_at'] else None
+            row = nodes.select(nodes.c.uuid == node['uuid']).execute().first()
+            self.assertEqual(
+                datetime.datetime.utcfromtimestamp(node['started_at']),
+                row['started_at'])
+            self.assertEqual(
+                finished_at,
+                row['finished_at'])
+
+    def _pre_upgrade_882b2d84cb1b(self, engine):
+        attributes = db_utils.get_table(engine, 'attributes')
+        nodes = db_utils.get_table(engine, 'nodes')
+        self.node_uuid = uuidutils.generate_uuid()
+        node = {
+            'uuid': self.node_uuid,
+            'started_at': datetime.datetime.utcnow(),
+            'finished_at': None,
+            'error': None,
+            'state': istate.States.starting
+        }
+        nodes.insert().values(node).execute()
+        data = {
+            'uuid': self.node_uuid,
+            'name': 'foo',
+            'value': 'bar'
+        }
+        attributes.insert().values(data).execute()
+
+    def _check_882b2d84cb1b(self, engine, data):
+        attributes = db_utils.get_table(engine, 'attributes')
+        col_names = [column.name for column in attributes.c]
+        self.assertIn('uuid', col_names)
+        self.assertIsInstance(attributes.c.uuid.type, sqlalchemy.types.String)
+        self.assertIn('node_uuid', col_names)
+        self.assertIsInstance(attributes.c.node_uuid.type,
+                              sqlalchemy.types.String)
+        self.assertIn('name', col_names)
+        self.assertIsInstance(attributes.c.name.type, sqlalchemy.types.String)
+        self.assertIn('value', col_names)
+        self.assertIsInstance(attributes.c.value.type, sqlalchemy.types.String)
+
+        row = attributes.select(attributes.c.node_uuid ==
+                                self.node_uuid).execute().first()
+        self.assertEqual(self.node_uuid, row.node_uuid)
+        self.assertNotEqual(self.node_uuid, row.uuid)
+        self.assertIsNotNone(row.uuid)
+        self.assertEqual('foo', row.name)
+        self.assertEqual('bar', row.value)
+
+    def _check_2970d2d44edc(self, engine, data):
+        nodes = db_utils.get_table(engine, 'nodes')
+        data = {'uuid': 'abcd'}
+        nodes.insert().execute(data)
+
+        n = nodes.select(nodes.c.uuid == 'abcd').execute().first()
+        self.assertIsNone(n['manage_boot'])
+
     def test_upgrade_and_version(self):
         with patch_with_engine(self.engine):
             self.migration_ext.upgrade('head')
