@@ -11,28 +11,30 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import base64
 import copy
-import functools
 import json
 import os
 import shutil
 import tempfile
-import time
 
 import eventlet
 import fixtures
 from ironicclient import exceptions
 import mock
 from oslo_config import cfg
+from oslo_serialization import base64
+from oslo_utils import timeutils
 from oslo_utils import uuidutils
+import six
 
 from ironic_inspector.common import ironic as ir_utils
-from ironic_inspector import firewall
+from ironic_inspector import db
+from ironic_inspector import introspection_state as istate
 from ironic_inspector import node_cache
 from ironic_inspector.plugins import base as plugins_base
 from ironic_inspector.plugins import example as example_plugin
 from ironic_inspector import process
+from ironic_inspector.pxe_filter import base as pxe_filter
 from ironic_inspector.test import base as test_base
 from ironic_inspector import utils
 
@@ -42,7 +44,7 @@ CONF = cfg.CONF
 class BaseTest(test_base.NodeTest):
     def setUp(self):
         super(BaseTest, self).setUp()
-        self.started_at = time.time()
+        self.started_at = timeutils.utcnow()
         self.all_ports = [mock.Mock(uuid=uuidutils.generate_uuid(),
                                     address=mac) for mac in self.macs]
         self.ports = [self.all_ports[1]]
@@ -65,12 +67,18 @@ class BaseProcessTest(BaseTest):
         self.find_mock = self.cache_fixture.mock
         self.node_info = node_cache.NodeInfo(
             uuid=self.node.uuid,
+            state=istate.States.waiting,
             started_at=self.started_at)
         self.node_info.finished = mock.Mock()
         self.find_mock.return_value = self.node_info
         self.cli.node.get.return_value = self.node
         self.process_mock = self.process_fixture.mock
         self.process_mock.return_value = self.fake_result_json
+        self.addCleanup(self._cleanup_lock, self.node_info)
+
+    def _cleanup_lock(self, node_info):
+        if node_info._locked:
+            node_info.release_lock()
 
 
 class TestProcess(BaseProcessTest):
@@ -85,7 +93,7 @@ class TestProcess(BaseProcessTest):
         self.assertEqual(sorted(self.all_macs), sorted(actual_macs))
         self.cli.node.get.assert_called_once_with(self.uuid)
         self.process_mock.assert_called_once_with(
-            self.node, self.data, self.node_info)
+            self.node_info, self.node, self.data)
 
     def test_no_ipmi(self):
         del self.inventory['bmc_address']
@@ -95,51 +103,77 @@ class TestProcess(BaseProcessTest):
         actual_macs = self.find_mock.call_args[1]['mac']
         self.assertEqual(sorted(self.all_macs), sorted(actual_macs))
         self.cli.node.get.assert_called_once_with(self.uuid)
-        self.process_mock.assert_called_once_with(self.node, self.data,
-                                                  self.node_info)
+        self.process_mock.assert_called_once_with(self.node_info, self.node,
+                                                  self.data)
+
+    def test_ipmi_not_detected(self):
+        self.inventory['bmc_address'] = '0.0.0.0'
+        process.process(self.data)
+
+        self.find_mock.assert_called_once_with(bmc_address=None, mac=mock.ANY)
+        actual_macs = self.find_mock.call_args[1]['mac']
+        self.assertEqual(sorted(self.all_macs), sorted(actual_macs))
+        self.cli.node.get.assert_called_once_with(self.uuid)
+        self.process_mock.assert_called_once_with(self.node_info, self.node,
+                                                  self.data)
+
+    def test_ipmi_not_detected_with_old_field(self):
+        self.inventory['bmc_address'] = '0.0.0.0'
+        self.data['ipmi_address'] = '0.0.0.0'
+        process.process(self.data)
+
+        self.find_mock.assert_called_once_with(bmc_address=None, mac=mock.ANY)
+        actual_macs = self.find_mock.call_args[1]['mac']
+        self.assertEqual(sorted(self.all_macs), sorted(actual_macs))
+        self.cli.node.get.assert_called_once_with(self.uuid)
+        self.process_mock.assert_called_once_with(self.node_info, self.node,
+                                                  self.data)
 
     def test_not_found_in_cache(self):
         self.find_mock.side_effect = utils.Error('not found')
-        self.assertRaisesRegexp(utils.Error,
-                                'not found',
-                                process.process, self.data)
+        self.assertRaisesRegex(utils.Error,
+                               'not found',
+                               process.process, self.data)
         self.assertFalse(self.cli.node.get.called)
         self.assertFalse(self.process_mock.called)
 
     def test_not_found_in_ironic(self):
         self.cli.node.get.side_effect = exceptions.NotFound()
 
-        self.assertRaisesRegexp(utils.Error,
-                                'Node %s was not found' % self.uuid,
-                                process.process, self.data)
+        self.assertRaisesRegex(utils.Error,
+                               'Node %s was not found' % self.uuid,
+                               process.process, self.data)
         self.cli.node.get.assert_called_once_with(self.uuid)
         self.assertFalse(self.process_mock.called)
-        self.node_info.finished.assert_called_once_with(error=mock.ANY)
+        self.node_info.finished.assert_called_once_with(
+            istate.Events.error, error=mock.ANY)
 
     def test_already_finished(self):
-        self.node_info.finished_at = time.time()
-        self.assertRaisesRegexp(utils.Error, 'already finished',
-                                process.process, self.data)
+        self.node_info.finished_at = timeutils.utcnow()
+        self.assertRaisesRegex(utils.Error, 'already finished',
+                               process.process, self.data)
         self.assertFalse(self.process_mock.called)
         self.assertFalse(self.find_mock.return_value.finished.called)
 
     def test_expected_exception(self):
         self.process_mock.side_effect = utils.Error('boom')
 
-        self.assertRaisesRegexp(utils.Error, 'boom',
-                                process.process, self.data)
+        self.assertRaisesRegex(utils.Error, 'boom',
+                               process.process, self.data)
 
-        self.node_info.finished.assert_called_once_with(error='boom')
+        self.node_info.finished.assert_called_once_with(
+            istate.Events.error, error='boom')
 
     def test_unexpected_exception(self):
         self.process_mock.side_effect = RuntimeError('boom')
 
-        with self.assertRaisesRegexp(utils.Error,
-                                     'Unexpected exception') as ctx:
+        with self.assertRaisesRegex(utils.Error,
+                                    'Unexpected exception') as ctx:
             process.process(self.data)
 
         self.assertEqual(500, ctx.exception.http_code)
         self.node_info.finished.assert_called_once_with(
+            istate.Events.error,
             error='Unexpected exception RuntimeError during processing: boom')
 
     def test_hook_unexpected_exceptions(self):
@@ -149,11 +183,11 @@ class TestProcess(BaseProcessTest):
             patcher.start()
             self.addCleanup(lambda p=patcher: p.stop())
 
-        self.assertRaisesRegexp(utils.Error, 'Unexpected exception',
-                                process.process, self.data)
+        self.assertRaisesRegex(utils.Error, 'Unexpected exception',
+                               process.process, self.data)
 
         self.node_info.finished.assert_called_once_with(
-            error=mock.ANY)
+            istate.Events.error, error=mock.ANY)
         error_message = self.node_info.finished.call_args[1]['error']
         self.assertIn('RuntimeError', error_message)
         self.assertIn('boom', error_message)
@@ -167,17 +201,17 @@ class TestProcess(BaseProcessTest):
             patcher.start()
             self.addCleanup(lambda p=patcher: p.stop())
 
-        self.assertRaisesRegexp(utils.Error, 'Unexpected exception',
-                                process.process, self.data)
+        self.assertRaisesRegex(utils.Error, 'Unexpected exception',
+                               process.process, self.data)
 
         self.assertFalse(self.node_info.finished.called)
 
     def test_error_if_node_not_found_hook(self):
         plugins_base._NOT_FOUND_HOOK_MGR = None
         self.find_mock.side_effect = utils.NotFoundInCacheError('BOOM')
-        self.assertRaisesRegexp(utils.Error,
-                                'Look up error: BOOM',
-                                process.process, self.data)
+        self.assertRaisesRegex(utils.Error,
+                               'Look up error: BOOM',
+                               process.process, self.data)
 
 
 @mock.patch.object(example_plugin, 'example_not_found_hook',
@@ -199,9 +233,9 @@ class TestNodeNotFoundHook(BaseProcessTest):
         plugins_base._NOT_FOUND_HOOK_MGR = None
         self.find_mock.side_effect = utils.NotFoundInCacheError('BOOM')
         hook_mock.return_value = None
-        self.assertRaisesRegexp(utils.Error,
-                                'Node not found hook returned nothing',
-                                process.process, self.data)
+        self.assertRaisesRegex(utils.Error,
+                               'Node not found hook returned nothing',
+                               process.process, self.data)
         hook_mock.assert_called_once_with(self.data)
 
     def test_node_not_found_hook_exception(self, hook_mock):
@@ -209,9 +243,9 @@ class TestNodeNotFoundHook(BaseProcessTest):
         plugins_base._NOT_FOUND_HOOK_MGR = None
         self.find_mock.side_effect = utils.NotFoundInCacheError('BOOM')
         hook_mock.side_effect = Exception('Hook Error')
-        self.assertRaisesRegexp(utils.Error,
-                                'Node not found hook failed: Hook Error',
-                                process.process, self.data)
+        self.assertRaisesRegex(utils.Error,
+                               'Node not found hook failed: Hook Error',
+                               process.process, self.data)
         hook_mock.assert_called_once_with(self.data)
 
 
@@ -256,7 +290,7 @@ class TestStoreLogs(BaseProcessTest):
         CONF.set_override('ramdisk_logs_dir', self.tempdir, 'processing')
 
         self.logs = b'test logs'
-        self.data['logs'] = base64.b64encode(self.logs)
+        self.data['logs'] = base64.encode_as_bytes(self.logs)
 
     def _check_contents(self, name=None):
         files = os.listdir(self.tempdir)
@@ -310,13 +344,18 @@ class TestStoreLogs(BaseProcessTest):
         process.process(self.data)
         self._check_contents()
 
+    @mock.patch.object(os, 'makedirs', autospec=True)
     @mock.patch.object(process.LOG, 'exception', autospec=True)
-    def test_failure_to_write(self, log_mock, hook_mock):
+    def test_failure_to_write(self, log_mock, makedirs_mock, hook_mock):
+        tempdir = tempfile.mkdtemp()
+        logs_dir = os.path.join(tempdir, 'I/never/exist')
         CONF.set_override('always_store_ramdisk_logs', True, 'processing')
-        CONF.set_override('ramdisk_logs_dir', '/I/cannot/write/here',
-                          'processing')
+        CONF.set_override('ramdisk_logs_dir', logs_dir, 'processing')
+        makedirs_mock.side_effect = OSError()
         process.process(self.data)
+        os.rmdir(tempdir)
         self.assertEqual([], os.listdir(self.tempdir))
+        self.assertTrue(makedirs_mock.called)
         self.assertTrue(log_mock.called)
 
     def test_directory_is_created(self, hook_mock):
@@ -344,16 +383,12 @@ class TestProcessNode(BaseTest):
                           'processing')
         self.validate_attempts = 5
         self.data['macs'] = self.macs  # validate_interfaces hook
+        self.valid_interfaces['eth3'] = {
+            'mac': self.macs[1], 'ip': self.ips[1], 'extra': {}, 'pxe': False
+        }
+        self.data['interfaces'] = self.valid_interfaces
         self.ports = self.all_ports
 
-        self.new_creds = ('user', 'password')
-        self.patch_credentials = [
-            {'op': 'add', 'path': '/driver_info/ipmi_username',
-             'value': self.new_creds[0]},
-            {'op': 'add', 'path': '/driver_info/ipmi_password',
-             'value': self.new_creds[1]},
-        ]
-
         self.cli.node.get_boot_device.side_effect = (
             [RuntimeError()] * self.validate_attempts + [None])
         self.cli.port.create.side_effect = self.ports
@@ -361,109 +396,71 @@ class TestProcessNode(BaseTest):
         self.cli.node.list_ports.return_value = []
 
         self.useFixture(fixtures.MockPatchObject(
-            firewall, 'update_filters', autospec=True))
+            pxe_filter, 'driver', autospec=True))
 
         self.useFixture(fixtures.MockPatchObject(
             eventlet.greenthread, 'sleep', autospec=True))
+        self.node_info._state = istate.States.waiting
+        db.Node(uuid=self.node_info.uuid, state=self.node_info._state,
+                started_at=self.node_info.started_at,
+                finished_at=self.node_info.finished_at,
+                error=self.node_info.error).save(self.session)
 
     def test_return_includes_uuid(self):
-        ret_val = process._process_node(self.node, self.data, self.node_info)
+        ret_val = process._process_node(self.node_info, self.node, self.data)
         self.assertEqual(self.uuid, ret_val.get('uuid'))
 
-    def test_return_includes_uuid_with_ipmi_creds(self):
-        self.node_info.set_option('new_ipmi_credentials', self.new_creds)
-        ret_val = process._process_node(self.node, self.data, self.node_info)
-        self.assertEqual(self.uuid, ret_val.get('uuid'))
-        self.assertTrue(ret_val.get('ipmi_setup_credentials'))
-
     @mock.patch.object(example_plugin.ExampleProcessingHook, 'before_update')
     def test_wrong_provision_state(self, post_hook_mock):
         self.node.provision_state = 'active'
 
         self.assertRaises(utils.Error, process._process_node,
-                          self.node, self.data, self.node_info)
+                          self.node_info, self.node, self.data)
         self.assertFalse(post_hook_mock.called)
 
     @mock.patch.object(example_plugin.ExampleProcessingHook, 'before_update')
     @mock.patch.object(node_cache.NodeInfo, 'finished', autospec=True)
     def test_ok(self, finished_mock, post_hook_mock):
-        process._process_node(self.node, self.data, self.node_info)
+        process._process_node(self.node_info, self.node, self.data)
 
         self.cli.port.create.assert_any_call(node_uuid=self.uuid,
-                                             address=self.macs[0])
+                                             address=self.macs[0],
+                                             extra={},
+                                             pxe_enabled=True)
         self.cli.port.create.assert_any_call(node_uuid=self.uuid,
-                                             address=self.macs[1])
+                                             address=self.macs[1],
+                                             extra={},
+                                             pxe_enabled=False)
         self.cli.node.set_power_state.assert_called_once_with(self.uuid, 'off')
         self.assertFalse(self.cli.node.validate.called)
 
         post_hook_mock.assert_called_once_with(self.data, self.node_info)
-        finished_mock.assert_called_once_with(mock.ANY)
+        finished_mock.assert_called_once_with(mock.ANY, istate.Events.finish)
 
     def test_port_failed(self):
         self.cli.port.create.side_effect = (
             [exceptions.Conflict()] + self.ports[1:])
 
-        process._process_node(self.node, self.data, self.node_info)
+        process._process_node(self.node_info, self.node, self.data)
 
         self.cli.port.create.assert_any_call(node_uuid=self.uuid,
-                                             address=self.macs[0])
+                                             address=self.macs[0],
+                                             extra={}, pxe_enabled=True)
         self.cli.port.create.assert_any_call(node_uuid=self.uuid,
-                                             address=self.macs[1])
-
-    def test_set_ipmi_credentials(self):
-        self.node_info.set_option('new_ipmi_credentials', self.new_creds)
-
-        process._process_node(self.node, self.data, self.node_info)
-
-        self.cli.node.update.assert_any_call(self.uuid, self.patch_credentials)
-        self.cli.node.set_power_state.assert_called_once_with(self.uuid, 'off')
-        self.cli.node.get_boot_device.assert_called_with(self.uuid)
-        self.assertEqual(self.validate_attempts + 1,
-                         self.cli.node.get_boot_device.call_count)
-
-    def test_set_ipmi_credentials_no_address(self):
-        self.node_info.set_option('new_ipmi_credentials', self.new_creds)
-        del self.node.driver_info['ipmi_address']
-        self.patch_credentials.append({'op': 'add',
-                                       'path': '/driver_info/ipmi_address',
-                                       'value': self.bmc_address})
-
-        process._process_node(self.node, self.data, self.node_info)
-
-        self.cli.node.update.assert_any_call(self.uuid, self.patch_credentials)
-        self.cli.node.set_power_state.assert_called_once_with(self.uuid, 'off')
-        self.cli.node.get_boot_device.assert_called_with(self.uuid)
-        self.assertEqual(self.validate_attempts + 1,
-                         self.cli.node.get_boot_device.call_count)
-
-    @mock.patch.object(node_cache.NodeInfo, 'finished', autospec=True)
-    def test_set_ipmi_credentials_timeout(self, finished_mock):
-        self.node_info.set_option('new_ipmi_credentials', self.new_creds)
-        self.cli.node.get_boot_device.side_effect = RuntimeError('boom')
-
-        process._process_node(self.node, self.data, self.node_info)
-
-        self.cli.node.update.assert_any_call(self.uuid, self.patch_credentials)
-        self.assertEqual(2, self.cli.node.update.call_count)
-        self.assertEqual(process._CREDENTIALS_WAIT_RETRIES,
-                         self.cli.node.get_boot_device.call_count)
-        self.assertFalse(self.cli.node.set_power_state.called)
-        finished_mock.assert_called_once_with(
-            mock.ANY,
-            error='Failed to validate updated IPMI credentials for node %s, '
-            'node might require maintenance' % self.uuid)
+                                             address=self.macs[1],
+                                             extra={}, pxe_enabled=False)
 
     @mock.patch.object(node_cache.NodeInfo, 'finished', autospec=True)
     def test_power_off_failed(self, finished_mock):
         self.cli.node.set_power_state.side_effect = RuntimeError('boom')
 
-        process._process_node(self.node, self.data, self.node_info)
+        process._process_node(self.node_info, self.node, self.data)
 
         self.cli.node.set_power_state.assert_called_once_with(self.uuid, 'off')
         finished_mock.assert_called_once_with(
-            mock.ANY,
+            mock.ANY, istate.Events.error,
             error='Failed to power off node %s, check its power '
-                  'management configuration: boom' % self.uuid
+            'management configuration: boom' % self.uuid
         )
 
     @mock.patch.object(example_plugin.ExampleProcessingHook, 'before_update')
@@ -472,19 +469,21 @@ class TestProcessNode(BaseTest):
         self.node.provision_state = 'enroll'
         self.node_info.node = mock.Mock(return_value=self.node)
 
-        process._process_node(self.node, self.data, self.node_info)
+        process._process_node(self.node_info, self.node, self.data)
 
         self.assertTrue(post_hook_mock.called)
         self.assertTrue(self.cli.node.set_power_state.called)
-        finished_mock.assert_called_once_with(self.node_info)
+        finished_mock.assert_called_once_with(
+            self.node_info, istate.Events.finish)
 
     @mock.patch.object(node_cache.NodeInfo, 'finished', autospec=True)
     def test_no_power_off(self, finished_mock):
         CONF.set_override('power_off', False, 'processing')
-        process._process_node(self.node, self.data, self.node_info)
+        process._process_node(self.node_info, self.node, self.data)
 
         self.assertFalse(self.cli.node.set_power_state.called)
-        finished_mock.assert_called_once_with(self.node_info)
+        finished_mock.assert_called_once_with(
+            self.node_info, istate.Events.finish)
 
     @mock.patch.object(process.swift, 'SwiftAPI', autospec=True)
     def test_store_data(self, swift_mock):
@@ -493,7 +492,7 @@ class TestProcessNode(BaseTest):
         name = 'inspector_data-%s' % self.uuid
         expected = self.data
 
-        process._process_node(self.node, self.data, self.node_info)
+        process._process_node(self.node_info, self.node, self.data)
 
         swift_conn.create_object.assert_called_once_with(name, mock.ANY)
         self.assertEqual(expected,
@@ -506,7 +505,7 @@ class TestProcessNode(BaseTest):
         name = 'inspector_data-%s' % self.uuid
         self.data['logs'] = 'something'
 
-        process._process_node(self.node, self.data, self.node_info)
+        process._process_node(self.node_info, self.node, self.data)
 
         swift_conn.create_object.assert_called_once_with(name, mock.ANY)
         self.assertNotIn('logs',
@@ -523,7 +522,7 @@ class TestProcessNode(BaseTest):
                   'value': name, 'op': 'add'}]
         expected = self.data
 
-        process._process_node(self.node, self.data, self.node_info)
+        process._process_node(self.node_info, self.node, self.data)
 
         swift_conn.create_object.assert_called_once_with(name, mock.ANY)
         self.assertEqual(expected,
@@ -535,11 +534,12 @@ class TestProcessNode(BaseTest):
 @mock.patch.object(node_cache, 'get_node', autospec=True)
 class TestReapply(BaseTest):
     def prepare_mocks(func):
-        @functools.wraps(func)
+        @six.wraps(func)
         def wrapper(self, pop_mock, *args, **kw):
             pop_mock.return_value = node_cache.NodeInfo(
                 uuid=self.node.uuid,
                 started_at=self.started_at)
+
             pop_mock.return_value.finished = mock.Mock()
             pop_mock.return_value.acquire_lock = mock.Mock()
             return func(self, pop_mock, *args, **kw)
@@ -563,12 +563,9 @@ class TestReapply(BaseTest):
     @prepare_mocks
     def test_locking_failed(self, pop_mock, reapply_mock):
         pop_mock.return_value.acquire_lock.return_value = False
-        exc = utils.Error('Node locked, please, try again later')
-
-        with self.assertRaises(type(exc)) as cm:
-            process.reapply(self.uuid)
-
-        self.assertEqual(str(exc), str(cm.exception))
+        self.assertRaisesRegex(utils.Error,
+                               'Node locked, please, try again later',
+                               process.reapply, self.uuid)
 
         pop_mock.assert_called_once_with(self.uuid, locked=False)
         pop_mock.return_value.acquire_lock.assert_called_once_with(
@@ -594,11 +591,18 @@ class TestReapplyNode(BaseTest):
                                              started_at=self.started_at,
                                              node=self.node)
         self.node_info.invalidate_cache = mock.Mock()
-        self.new_creds = ('user', 'password')
 
         self.cli.port.create.side_effect = self.ports
         self.cli.node.update.return_value = self.node
         self.cli.node.list_ports.return_value = []
+        self.node_info._state = istate.States.finished
+        self.commit_fixture = self.useFixture(
+            fixtures.MockPatchObject(node_cache.NodeInfo, 'commit',
+                                     autospec=True))
+        db.Node(uuid=self.node_info.uuid, state=self.node_info._state,
+                started_at=self.node_info.started_at,
+                finished_at=self.node_info.finished_at,
+                error=self.node_info.error).save(self.session)
 
     def call(self):
         process._reapply(self.node_info)
@@ -606,7 +610,7 @@ class TestReapplyNode(BaseTest):
         self.node_info.release_lock.assert_called_once_with(self.node_info)
 
     def prepare_mocks(fn):
-        @functools.wraps(fn)
+        @six.wraps(fn)
         def wrapper(self, release_mock, finished_mock, swift_mock,
                     *args, **kw):
             finished_mock.side_effect = lambda *a, **kw: \
@@ -616,17 +620,13 @@ class TestReapplyNode(BaseTest):
         return wrapper
 
     @prepare_mocks
-    def test_ok(self, finished_mock, swift_mock, apply_mock,
-                post_hook_mock):
+    def test_ok(self, finished_mock, swift_mock, apply_mock, post_hook_mock):
         swift_name = 'inspector_data-%s' % self.uuid
         swift_mock.get_object.return_value = json.dumps(self.data)
 
-        with mock.patch.object(process.LOG, 'error',
-                               autospec=True) as log_mock:
-            self.call()
+        self.call()
 
-        # no failures logged
-        self.assertFalse(log_mock.called)
+        self.commit_fixture.mock.assert_called_once_with(self.node_info)
 
         post_hook_mock.assert_called_once_with(mock.ANY, self.node_info)
         swift_mock.create_object.assert_called_once_with(swift_name,
@@ -638,7 +638,8 @@ class TestReapplyNode(BaseTest):
 
         # assert no power operations were performed
         self.assertFalse(self.cli.node.set_power_state.called)
-        finished_mock.assert_called_once_with(self.node_info)
+        finished_mock.assert_called_once_with(
+            self.node_info, istate.Events.finish)
 
         # asserting validate_interfaces was called
         self.assertEqual(self.pxe_interfaces, swifted_data['interfaces'])
@@ -648,32 +649,30 @@ class TestReapplyNode(BaseTest):
         # behind validate_interfaces
         self.cli.port.create.assert_called_once_with(
             node_uuid=self.uuid,
-            address=swifted_data['macs'][0]
+            address=swifted_data['macs'][0],
+            extra={},
+            pxe_enabled=True
         )
 
     @prepare_mocks
-    def test_get_incomming_data_exception(self, finished_mock,
-                                          swift_mock, apply_mock,
-                                          post_hook_mock, ):
+    def test_get_incomming_data_exception(self, finished_mock, swift_mock,
+                                          apply_mock, post_hook_mock):
         exc = Exception('Oops')
+        expected_error = ('Unexpected exception Exception while fetching '
+                          'unprocessed introspection data from Swift: Oops')
         swift_mock.get_object.side_effect = exc
-        with mock.patch.object(process.LOG, 'exception',
-                               autospec=True) as log_mock:
-            self.call()
-
-        log_mock.assert_called_once_with('Encountered exception '
-                                         'while fetching stored '
-                                         'introspection data',
-                                         node_info=self.node_info)
+        self.call()
 
+        self.commit_fixture.mock.assert_called_once_with(self.node_info)
         self.assertFalse(swift_mock.create_object.called)
         self.assertFalse(apply_mock.called)
         self.assertFalse(post_hook_mock.called)
-        self.assertFalse(finished_mock.called)
+        finished_mock.assert_called_once_with(
+            self.node_info, istate.Events.error, error=expected_error)
 
     @prepare_mocks
-    def test_prehook_failure(self, finished_mock, swift_mock,
-                             apply_mock, post_hook_mock, ):
+    def test_prehook_failure(self, finished_mock, swift_mock, apply_mock,
+                             post_hook_mock):
         CONF.set_override('processing_hooks', 'example',
                           'processing')
         plugins_base._HOOKS_MGR = None
@@ -684,42 +683,31 @@ class TestReapplyNode(BaseTest):
         with mock.patch.object(example_plugin.ExampleProcessingHook,
                                'before_processing') as before_processing_mock:
             before_processing_mock.side_effect = exc
-            with mock.patch.object(process.LOG, 'error',
-                                   autospec=True) as log_mock:
-                self.call()
+            self.call()
 
-        exc_failure = ('Unexpected exception %(exc_class)s during '
+        exc_failure = ('Pre-processing failures detected reapplying '
+                       'introspection on stored data:\n'
+                       'Unexpected exception %(exc_class)s during '
                        'preprocessing in hook example: %(error)s' %
                        {'exc_class': type(exc).__name__, 'error':
                         exc})
-        log_mock.assert_called_once_with('Pre-processing failures '
-                                         'detected reapplying '
-                                         'introspection on stored '
-                                         'data:\n%s', exc_failure,
-                                         node_info=self.node_info)
-        finished_mock.assert_called_once_with(self.node_info,
-                                              error=exc_failure)
+        finished_mock.assert_called_once_with(
+            self.node_info, istate.Events.error, error=exc_failure)
         # assert _reapply ended having detected the failure
         self.assertFalse(swift_mock.create_object.called)
         self.assertFalse(apply_mock.called)
         self.assertFalse(post_hook_mock.called)
 
     @prepare_mocks
-    def test_generic_exception_creating_ports(self, finished_mock,
-                                              swift_mock, apply_mock,
-                                              post_hook_mock):
+    def test_generic_exception_creating_ports(self, finished_mock, swift_mock,
+                                              apply_mock, post_hook_mock):
         swift_mock.get_object.return_value = json.dumps(self.data)
         exc = Exception('Oops')
         self.cli.port.create.side_effect = exc
+        self.call()
 
-        with mock.patch.object(process.LOG, 'exception') as log_mock:
-            self.call()
-
-        log_mock.assert_called_once_with('Encountered exception reapplying'
-                                         ' introspection on stored data',
-                                         node_info=self.node_info,
-                                         data=mock.ANY)
-        finished_mock.assert_called_once_with(self.node_info, error=str(exc))
+        finished_mock.assert_called_once_with(
+            self.node_info, istate.Events.error, error=str(exc))
         self.assertFalse(swift_mock.create_object.called)
         self.assertFalse(apply_mock.called)
         self.assertFalse(post_hook_mock.called)
