@@ -11,18 +11,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import datetime
 import logging as pylog
-import re
 
 import futurist
+from ironicclient.v1 import node
 from keystonemiddleware import auth_token
 from oslo_config import cfg
 from oslo_log import log
 from oslo_middleware import cors as cors_middleware
-import six
+import pytz
 
-from ironic_inspector.common.i18n import _, _LE
-from ironic_inspector import conf  # noqa
+from ironic_inspector.common.i18n import _
+from ironic_inspector import policy
 
 CONF = cfg.CONF
 
@@ -31,9 +32,15 @@ _EXECUTOR = None
 
 def get_ipmi_address_from_data(introspection_data):
     try:
-        return introspection_data['inventory']['bmc_address']
+        result = introspection_data['inventory']['bmc_address']
     except KeyError:
-        return introspection_data.get('ipmi_address')
+        result = introspection_data.get('ipmi_address')
+
+    if result in ('', '0.0.0.0'):
+        # ipmitool can return these values, if it does not know the address
+        return None
+    else:
+        return result
 
 
 def get_pxe_mac(introspection_data):
@@ -49,7 +56,7 @@ def processing_logger_prefix(data=None, node_info=None):
     """Calculate prefix for logging.
 
     Tries to use:
-    * node UUID,
+    * node UUID, node._state
     * node PXE MAC,
     * node BMC address
 
@@ -62,16 +69,18 @@ def processing_logger_prefix(data=None, node_info=None):
     data = data or {}
 
     if node_info is not None:
-        parts.append(str(node_info.uuid))
+        if isinstance(node_info, node.Node):
+            parts.append(str(node_info.uuid))
+        else:
+            parts.append(str(node_info))
 
     pxe_mac = get_pxe_mac(data)
     if pxe_mac:
         parts.append('MAC %s' % pxe_mac)
 
-    if CONF.processing.log_bmc_address:
-        bmc_address = get_ipmi_address_from_data(data) if data else None
-        if bmc_address:
-            parts.append('BMC %s' % bmc_address)
+    bmc_address = get_ipmi_address_from_data(data) if data else None
+    if bmc_address:
+        parts.append('BMC %s' % bmc_address)
 
     if parts:
         return _('[node: %s]') % ' '.join(parts)
@@ -113,9 +122,22 @@ class Error(Exception):
 class NotFoundInCacheError(Error):
     """Exception when node was not found in cache during processing."""
 
-    def __init__(self, msg, code=404):
+    def __init__(self, msg, code=404, **kwargs):
         super(NotFoundInCacheError, self).__init__(msg, code,
-                                                   log_level='info')
+                                                   log_level='info', **kwargs)
+
+
+class NodeStateRaceCondition(Error):
+    """State mismatch between the DB and a node_info."""
+    def __init__(self, *args, **kwargs):
+        message = _('Node state mismatch detected between the DB and the '
+                    'cached node_info object')
+        kwargs.setdefault('code', 500)
+        super(NodeStateRaceCondition, self).__init__(message, *args, **kwargs)
+
+
+class NodeStateInvalidEvent(Error):
+    """Invalid event attempted."""
 
 
 def executor():
@@ -133,29 +155,6 @@ def add_auth_middleware(app):
     :param app: application.
     """
     auth_conf = dict(CONF.keystone_authtoken)
-    # These items should only be used for accessing Ironic API.
-    # For keystonemiddleware's authentication,
-    # keystone_authtoken's items will be used and
-    # these items will be unsupported.
-    # [ironic]/os_password
-    # [ironic]/os_username
-    # [ironic]/os_auth_url
-    # [ironic]/os_tenant_name
-    auth_conf.update({'admin_password':
-                      CONF.ironic.os_password or
-                      CONF.keystone_authtoken.admin_password,
-                      'admin_user':
-                      CONF.ironic.os_username or
-                      CONF.keystone_authtoken.admin_user,
-                      'auth_uri':
-                      CONF.ironic.os_auth_url or
-                      CONF.keystone_authtoken.auth_uri,
-                      'admin_tenant_name':
-                      CONF.ironic.os_tenant_name or
-                      CONF.keystone_authtoken.admin_tenant_name,
-                      'identity_uri':
-                      CONF.ironic.identity_uri or
-                      CONF.keystone_authtoken.identity_uri})
     auth_conf['delay_auth_decision'] = True
     app.wsgi_app = auth_token.AuthProtocol(app.wsgi_app, auth_conf)
 
@@ -171,33 +170,21 @@ def add_cors_middleware(app):
     app.wsgi_app = cors_middleware.CORS(app.wsgi_app, CONF)
 
 
-def check_auth(request):
+def check_auth(request, rule=None, target=None):
     """Check authentication on request.
 
     :param request: Flask request
+    :param rule: policy rule to check the request against
     :raises: utils.Error if access is denied
     """
-    if get_auth_strategy() == 'noauth':
+    if CONF.auth_strategy == 'noauth':
         return
-    if request.headers.get('X-Identity-Status').lower() == 'invalid':
-        raise Error(_('Authentication required'), code=401)
-    roles = (request.headers.get('X-Roles') or '').split(',')
-    if 'admin' not in roles:
-        LOG.error(_LE('Role "admin" not in user role list %s'), roles)
-        raise Error(_('Access denied'), code=403)
-
-
-def is_valid_mac(address):
-    """Return whether given value is a valid MAC."""
-    m = "[0-9a-f]{2}(:[0-9a-f]{2}){5}$"
-    return (isinstance(address, six.string_types)
-            and re.match(m, address.lower()))
-
-
-def get_auth_strategy():
-    if CONF.authenticate is not None:
-        return 'keystone' if CONF.authenticate else 'noauth'
-    return CONF.auth_strategy
+    if not request.context.is_public_api:
+        if request.headers.get('X-Identity-Status', '').lower() == 'invalid':
+            raise Error(_('Authentication required'), code=401)
+    target = {} if target is None else target
+    if not policy.authorize(rule, target, request.context.to_policy_values()):
+        raise Error(_("Access denied by policy"), code=403)
 
 
 def get_valid_macs(data):
@@ -207,7 +194,7 @@ def get_valid_macs(data):
             if m.get('mac')]
 
 
-_INVENTORY_MANDATORY_KEYS = ('disks', 'memory', 'cpu', 'interfaces')
+_INVENTORY_MANDATORY_KEYS = ('memory', 'cpu', 'interfaces')
 
 
 def get_inventory(data, node_info=None):
@@ -223,4 +210,23 @@ def get_inventory(data, node_info=None):
             raise Error(_('Invalid hardware inventory: %s key is missing '
                           'or empty') % key, data=data, node_info=node_info)
 
+    if not inventory.get('disks'):
+        LOG.info('No disks were detected in the inventory, assuming this '
+                 'is a disk-less node', data=data, node_info=node_info)
+        # Make sure the code iterating over it does not fail with a TypeError
+        inventory['disks'] = []
+
     return inventory
+
+
+def iso_timestamp(timestamp=None, tz=pytz.timezone('utc')):
+    """Return an ISO8601-formatted timestamp (tz: UTC) or None.
+
+    :param timestamp: such as time.time() or None
+    :param tz: timezone
+    :returns: an ISO8601-formatted timestamp, or None
+    """
+    if timestamp is None:
+        return None
+    date = datetime.datetime.fromtimestamp(timestamp, tz=tz)
+    return date.isoformat()
